require 'json'
require 'ua_dict/version'

module UaDict

  #
  # @browser_data   浏览器版本和制造商的关系数据
  # @operating_data 操作系统和制造商的关系数据
  # @device_data    设备和制造商的关系数据
  #

  def initialize_parser
    @ua_dict_path   = File.dirname(__FILE__) + '/ua_dict/dict/'

    @browser_data   = ua_data('Browser.json')
    @operating_data = ua_data('OperatingSystem.json')
    @device_data    = ua_data('Device.json')

    @mobile         = false
  end

  def ua_data json_file
    result = []
    File.open(@ua_dict_path + json_file) do |f|
      result = JSON.load(f)
    end
    raise "Parse error: #{json_file}" if result == []
    result
  end

  def fetch_aliases op, ua
    op['aliases'].each do |_alias|
      if ua.upcase.include?(_alias.upcase)
        return op, _alias
      end
    end
    nil
  end

  def fetch_info type_data, ua
    found, info = false, nil
    type_data.each do |ua_dict|
      has_res = fetch_aliases(ua_dict, ua)
      if has_res
        found, info = true, has_res
        if ua_dict['children'] != []
          found, info = fetch_info(ua_dict['children'], ua)
        end
        break if found
      end
    end
    return found, info
  end

  def decode_ua ua
    @b_info = fetch_info(@browser_data, ua)[1]
    if ua.upcase.include?('MOBILE')
      @mobile = true
      @d_info = fetch_info(@device_data, ua)[1]
    end
    @o_info = fetch_info(@operating_data, ua)[1]
  end

  # [name, alias]
  def brower_info
    if @b_info
      [@b_info[0]['name'], @b_info[1]]
    else
      ['Unknown', 'Unknown']
    end
  end

  # [name, alias]
  def device_info
    if @d_info
      [@d_info[0]['name'], @d_info[1]]
    else
      ['Unknown', 'Unknown']
    end
  end

  # [name, alias]
  def operating_info
    if @o_info
      [@o_info[0]['name'], @o_info[1]]
    else
      ['Unknown', 'Unknown']
    end
  end

  def is_mobile?
    @mobile
  end
end